@@ -54,6 +54,7 @@ static int     v4l2_flags(void *handle);
 static struct ng_attribute* v4l2_attrs(void *handle);
 static int     v4l2_read_attr(struct ng_attribute*);
 static void    v4l2_write_attr(struct ng_attribute*, int val);
+static void    v4l2_get_min_size(void *hdl, int *min_width, int *min_height);
 
 /* overlay */
 static int   v4l2_setupfb(void *handle, struct ng_video_fmt *fmt, void *base);
@@ -86,7 +87,9 @@ struct v4l2_handle {
     int                         fd;
 
     /* device descriptions */
-    int                         ninputs,nstds,nfmts;
+    char                        *device;
+    int                         ninputs, nstds, nfmts, read_done;
+    unsigned int                min_width, min_height;
     struct v4l2_capability	cap;
     struct v4l2_streamparm	streamparm;
     struct v4l2_input		inp[MAX_INPUT];
@@ -131,6 +134,7 @@ struct ng_vid_driver v4l2_driver = {
     get_devname:   v4l2_devname,
     capabilities:  v4l2_flags,
     list_attrs:    v4l2_attrs,
+    get_min_size:  v4l2_get_min_size,
 
     setupfb:       v4l2_setupfb,
     overlay:       v4l2_overlay,
@@ -263,6 +267,41 @@ get_device_capabilities(struct v4l2_handle *h)
     }
 }
 
+static void
+find_min_size(struct v4l2_handle *h)
+{
+    int i;
+    struct v4l2_fmtdesc fmtdesc = { .type = V4L2_BUF_TYPE_VIDEO_CAPTURE };
+    struct v4l2_format fmt = { .type = V4L2_BUF_TYPE_VIDEO_CAPTURE };
+
+    if (xioctl(h->fd, VIDIOC_G_FMT, &fmt, 0)) {
+        h->min_width  = 32;
+        h->min_height = 24;
+        return;
+    }
+
+    h->min_width = -1;
+    h->min_height = -1;
+
+    for (i = 0; ; i++) {
+        fmtdesc.index = i;
+
+        if (xioctl(h->fd, VIDIOC_ENUM_FMT, &fmtdesc, 1))
+            break;
+
+        fmt.fmt.pix.pixelformat = fmtdesc.pixelformat;
+        fmt.fmt.pix.width = 32;
+        fmt.fmt.pix.height = 24;
+
+        if (xioctl(h->fd, VIDIOC_TRY_FMT, &fmt, 0) == 0) {
+            if (fmt.fmt.pix.width < h->min_width)
+                h->min_width = fmt.fmt.pix.width;
+            if (fmt.fmt.pix.height < h->min_height)
+                h->min_height = fmt.fmt.pix.height;
+        }
+    }
+}
+
 static struct STRTAB *
 build_norms(struct v4l2_handle *h)
 {
@@ -507,6 +546,11 @@ v4l2_open_handle(char *device, int req_flags)
     if (NULL == h)
 	return NULL;
     memset(h,0,sizeof(*h));
+    h->device = strdup(device);
+    if (!h->device) {
+        free(h);
+	return NULL;
+    }
 
     if (-1 == (h->fd = open(device, O_RDWR))) {
 #ifdef HAVE_EXPLAIN
@@ -552,6 +596,10 @@ v4l2_open_handle(char *device, int req_flags)
 		h->cap.version         & 0xff,
 		h->cap.card,h->cap.bus_info);
     get_device_capabilities(h);
+    find_min_size(h);
+    if (ng_debug)
+	fprintf(stderr,"v4l2: device min size %ux%u\n",
+	        h->min_width, h->min_height);
 
     /* attributes */
     v4l2_add_attr(h, NULL, ATTR_ID_NORM,  build_norms(h));
@@ -611,6 +659,7 @@ v4l2_close_handle(void *handle)
 	h->attr = NULL;
     }
 
+    free(h->device);
     free(h);
     h = NULL;
 
@@ -644,6 +693,13 @@ static struct ng_attribute* v4l2_attrs(void *handle)
     return h->attr;
 }
 
+static void v4l2_get_min_size(void *handle, int *min_width, int *min_height)
+{
+    struct v4l2_handle *h = handle;
+    *min_width = h->min_width;
+    *min_height = h->min_height;
+}
+
 /* ---------------------------------------------------------------------- */
 
 static unsigned long
@@ -1016,7 +1072,9 @@ static int
 v4l2_setformat(void *handle, struct ng_video_fmt *fmt)
 {
     struct v4l2_handle *h = handle;
+    int rc, fd;
 
+retry:
     h->fmt_v4l2.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
     h->fmt_v4l2.fmt.pix.pixelformat  = xawtv_pixelformat[fmt->fmtid];
     h->fmt_v4l2.fmt.pix.width        = fmt->width;
@@ -1028,10 +1086,58 @@ v4l2_setformat(void *handle, struct ng_video_fmt *fmt)
     else
 	h->fmt_v4l2.fmt.pix.bytesperline = 0;
 
-    if (-1 == xioctl(h->fd, VIDIOC_S_FMT, &h->fmt_v4l2, 1))
-	return -1;
+#ifdef USE_LIBV4L
+    rc = v4l2_ioctl(h->fd, VIDIOC_S_FMT, &h->fmt_v4l2);
+#else
+    rc = ioctl(h->fd, VIDIOC_S_FMT, &h->fmt_v4l2);
+#endif
+    /* Some devices do not like mixing read and mmap, they give EBUSY
+       here after the first read */
+    if (rc < 0 && errno == EBUSY && h->read_done) {
+        fprintf(stderr, "v4l2: %s does not support switching between "
+                        "read and mmap, reopening\n", h->device);
+        /* HACK only way to recover is to close and re-open the fd */
+        fd = open(h->device, O_RDWR);
+        if (fd == -1) {
+            fprintf(stderr, "v4l2: open %s: %s\n", h->device, strerror(errno));
+            return -1;
+        }
+#ifdef USE_LIBV4L
+        rc = v4l2_fd_open(fd, 0);
+        if (rc != -1)
+	    fd = rc;
+        v4l2_close(h->fd);
+#else
+        close(h->fd);
+#endif
+        h->fd = fd;
+        h->cap.capabilities &= ~V4L2_CAP_READWRITE;
+        h->read_done = 0;
+        goto retry;
+    }
+    if (rc < 0) {
+        print_ioctl(stderr, ioctls_v4l2, PREFIX, VIDIOC_S_FMT, &h->fmt_v4l2);
+        fprintf(stderr,": %s\n", strerror(errno));
+        return -1;
+    }
     if (h->fmt_v4l2.fmt.pix.pixelformat != xawtv_pixelformat[fmt->fmtid])
 	return -1;
+
+    /*
+     * The bttv driver has a bug where it will return a width which is not
+     * a multiple of 16 for planar formats, while it cannot handle this,
+     * fix this up.
+     *
+     * A kernel fix has been send upstream for this for 4.5 / 4.6, so
+     * eventually this workaround should be removed.
+     */
+    if (!strcmp(h->cap.driver, "bttv") &&
+            (fmt->fmtid == VIDEO_YUV422P || fmt->fmtid == VIDEO_YUV420P) &&
+            h->fmt_v4l2.fmt.pix.width % 16) {
+        fmt->width = h->fmt_v4l2.fmt.pix.width & ~15;
+        goto retry;
+    }
+
     fmt->width        = h->fmt_v4l2.fmt.pix.width;
     fmt->height       = h->fmt_v4l2.fmt.pix.height;
     fmt->bytesperline = h->fmt_v4l2.fmt.pix.bytesperline;
@@ -1151,7 +1257,19 @@ v4l2_getimage(void *handle)
 
     size = h->fmt_me.bytesperline * h->fmt_me.height;
     buf = ng_malloc_video_buf(&h->fmt_me,size);
-    if (h->cap.capabilities & V4L2_CAP_READWRITE) {
+    if (h->cap.capabilities & V4L2_CAP_STREAMING) {
+	if (-1 == v4l2_start_streaming(h,1)) {
+	    v4l2_stop_streaming(h);
+	    return NULL;
+	}
+	frame = v4l2_waiton(h);
+	if (-1 == frame) {
+	    v4l2_stop_streaming(h);
+	    return NULL;
+	}
+	memcpy(buf->data,h->buf_me[0].data,size);
+	v4l2_stop_streaming(h);
+    } else {
 #ifndef USE_LIBV4L
 	rc = read(h->fd,buf->data,size);
 #else /* USE_LIBV4L */
@@ -1168,6 +1286,8 @@ v4l2_getimage(void *handle)
 	    h->ov_on = 1;
 	    xioctl(h->fd, VIDIOC_OVERLAY, &h->ov_on, 0);
 	}
+        if (rc >= 0)
+            h->read_done = 1;
 	if (rc != size) {
 	    if (-1 == rc) {
 		perror("v4l2: read");
@@ -1177,18 +1297,6 @@ v4l2_getimage(void *handle)
 	    ng_release_video_buf(buf);
 	    return NULL;
 	}
-    } else {
-	if (-1 == v4l2_start_streaming(h,1)) {
-	    v4l2_stop_streaming(h);
-	    return NULL;
-	}
-	frame = v4l2_waiton(h);
-	if (-1 == frame) {
-	    v4l2_stop_streaming(h);
-	    return NULL;
-	}
-	memcpy(buf->data,h->buf_me[0].data,size);
-	v4l2_stop_streaming(h);
     }
     return buf;
 }
