@@ -22,8 +22,9 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
-#ifdef __linux__
-# include "videodev.h"
+#if 0 /* def __linux__ */
+#include <linux/types.h>
+# include "videodev2.h"
 #endif
 
 #include "httpd.h"
@@ -104,15 +105,15 @@ usage(char *name)
 	    "  -l log   write access log to file >log<      [%s]\n"
 	    "  -L log   same as above + flush every line\n"
 #if 0
-            "  -r       poll tv frequency and clear cache\n"
+	    "  -r       poll tv frequency and clear cache\n"
 	    "           on station changes                  [%s]\n"
 	    "  -a       use ascii art for block graphics    [%s]\n"
 #endif
 	    "",
 	    h ? h+1 : name,
 	    ng_dev.vbi,
- 	    debug     ?  "on" : "off",
- 	    dontdetach ?  "on" : "off",
+	    debug     ?  "on" : "off",
+	    dontdetach ?  "on" : "off",
 	    usesyslog ?  "on" : "off",
 	    timeout, max_conn,
 	    listen_port,
@@ -141,7 +142,7 @@ fix_ug(void)
 {
     struct passwd  *pw = NULL;
     struct group   *gr = NULL;
-    
+
     /* root is allowed to use any uid/gid,
      * others will get their real uid/gid */
     if (0 == getuid() && strlen(user) > 0) {
@@ -167,8 +168,10 @@ fix_ug(void)
     }
 
     /* set group */
-    if (getegid() != gr->gr_gid || getgid() != gr->gr_gid)
+    if (getegid() != gr->gr_gid || getgid() != gr->gr_gid) {
+        setgroups(0, NULL);
 	setgid(gr->gr_gid);
+    }
     if (getegid() != gr->gr_gid || getgid() != gr->gr_gid) {
 	xerror(LOG_ERR,"setgid failed",NULL);
 	exit(1);
@@ -176,8 +179,10 @@ fix_ug(void)
     strncpy(group,gr->gr_name,16);
 
     /* set user */
-    if (geteuid() != pw->pw_uid || getuid() != pw->pw_uid)
+    if (geteuid() != pw->pw_uid || getuid() != pw->pw_uid) {
+        setgroups(0, NULL);
 	setuid(pw->pw_uid);
+    }
     if (geteuid() != pw->pw_uid || getuid() != pw->pw_uid) {
 	xerror(LOG_ERR,"setuid failed",NULL);
 	exit(1);
@@ -286,7 +291,7 @@ xerror(int loglevel, char *txt, char *peerhost)
 	    syslog(loglevel,"%s\n",txt);
 	else
 	    syslog(loglevel,"%s (peer=%s)\n",txt,peerhost);
-    }	
+    }
 }
 
 static void
@@ -549,7 +554,7 @@ main(int argc, char *argv[])
 	if (res->ai_canonname)
 	    strcpy(server_host,res->ai_canonname);
     }
-    
+
     /* parse options */
     v4 = 1; v6 = 1; simulate = 0;
     for (;;) {
@@ -586,9 +591,9 @@ main(int argc, char *argv[])
 	case 'r':
 	    cachereset++;
 	    break;
-        case 'N':
-            canonicalhost = 1;
-            /* fall through */
+	case 'N':
+	    canonicalhost = 1;
+	    /* fall through */
 	case 'n':
 	    strncpy(server_host,optarg,sizeof(server_host)-1);
 	    break;
@@ -657,7 +662,7 @@ main(int argc, char *argv[])
 		xperror(LOG_ERR,"socket (ipv6)",NULL);
 	}
     }
-    
+
     /* ... failing that try ipv4 */
     if (-1 == slisten  &&  v4) {
 	ask.ai_family = PF_INET;
@@ -691,11 +696,11 @@ main(int argc, char *argv[])
 
     if (-1 == bind(slisten, (struct sockaddr*) &ss, ss_len)) {
 	xperror(LOG_ERR,"bind",NULL);
-        exit(1);
+	exit(1);
     }
     if (-1 == listen(slisten, 2*max_conn)) {
 	xperror(LOG_ERR,"listen",NULL);
-        exit(1);
+	exit(1);
     }
 
     /* change user/group - also does chroot */
@@ -725,17 +730,17 @@ main(int argc, char *argv[])
 
     /* run as daemon - detach from terminal */
     if ((!debug) && (!dontdetach)) {
-        switch (fork()) {
-        case -1:
+	switch (fork()) {
+	case -1:
 	    xperror(LOG_ERR,"fork",NULL);
 	    exit(1);
-        case 0:
-            close(0); close(1); close(2); setsid();
+	case 0:
+	    close(0); close(1); close(2); setsid();
 	    have_tty = 0;
-            break;
-        default:
-            exit(0);
-        }
+	    break;
+	default:
+	    exit(0);
+	}
     }
     if (usesyslog) {
 	syslog_start();
@@ -757,7 +762,7 @@ main(int argc, char *argv[])
     /* go! */
     start = time(NULL);
     mainloop();
-    
+
     if (logfd)
 	fclose(logfd);
     if (debug)
