@@ -65,6 +65,10 @@
 
 /*----------------------------------------------------------------------*/
 
+#define ALSA_LATENCY_DEFAULT 30
+#define STR(X) __STR(X)
+#define __STR(X) #X
+
 XtAppContext      app_context;
 Widget            app_shell, tv;
 Widget            on_shell;
@@ -99,6 +103,10 @@ XRRScreenSize      *randr;
 int                nrandr;
 int                randr_evbase;
 #endif
+#if defined(HAVE_ALSA)
+static char        *alsa_cap;
+static char        *alsa_out;
+#endif
 
 static Widget on_label;
 static XtIntervalId title_timer, on_timer;
@@ -163,7 +171,7 @@ XtResource args_desc[] = {
 	"alsa_latency",
 	XtCValue, XtRInt, sizeof(int),
 	XtOffset(struct ARGS*,alsa_latency),
-	XtRString, "30"
+	XtRString, STR(ALSA_LATENCY_DEFAULT)
     },{
 	/* Integer */
 	"debug",
@@ -1292,25 +1300,45 @@ FilterAction(Widget widget, XEvent *event,
 static int xfree_dga_error_base;
 #endif
 
+#ifdef HAVE_LIBXXF86DGA
+static int (*orig_xfree_error_handler)(Display *, XErrorEvent *);
+
+static int xfree_dga_error_handler(Display *d, XErrorEvent *e)
+{
+  if (e->error_code == (xfree_dga_error_base + XF86DGANoDirectVideoMode) ||
+      e->error_code == (xfree_dga_error_base + XF86DGAClientNotLocal)) {
+    have_dga = 0;
+    return 0;
+  }
+  return orig_xfree_error_handler(d, e);
+}
+#endif
+
 void
 xfree_dga_init(Display *dpy)
 {
 #ifdef HAVE_LIBXXF86DGA
-    int  flags,foo,ma,mi;
+    int  flags = 0,foo, ma, mi;
 
     if (!do_overlay)
 	return;
 
     if (args.dga) {
+	have_dga = 1;
+	orig_xfree_error_handler = XSetErrorHandler(xfree_dga_error_handler);
 	if (XF86DGAQueryExtension(dpy,&foo,&xfree_dga_error_base)) {
+	    if (!have_dga)
+		return;
 	    XF86DGAQueryDirectVideo(dpy,XDefaultScreen(dpy),&flags);
 	    if (flags & XF86DGADirectPresent) {
 		XF86DGAQueryVersion(dpy,&ma,&mi);
 		if (debug)
 		    fprintf(stderr,"DGA version %d.%d\n",ma,mi);
-		have_dga = 1;
+	    } else {
+		have_dga = 0;
 	    }
 	}
+	XSetErrorHandler(orig_xfree_error_handler);
     }
 #endif
 }
@@ -1392,16 +1420,20 @@ xfree_xinerama_init(Display *dpy)
 #endif
 }
 
-#ifdef HAVE_LIBXXF86DGA
-static int (*orig_xfree_error_handler)(Display *, XErrorEvent *);
-
-static int xfree_dga_error_handler(Display *d, XErrorEvent *e)
+#if defined(HAVE_ALSA)
+static void x11_mute_notify(int val)
 {
-  if (e->error_code == (xfree_dga_error_base + XF86DGANoDirectVideoMode)) {
-    have_dga = 0;
-    return 0;
-  }
-  return orig_xfree_error_handler(d, e);
+    if (val)
+	alsa_thread_stop();
+    else if (!alsa_thread_is_running()) {
+        if (args.readconfig && args.alsa_latency == ALSA_LATENCY_DEFAULT) {
+            int val = cfg_get_int("global", "alsa_latency");
+            if (val > 0)
+                args.alsa_latency = val;
+        }
+	alsa_thread_startup(alsa_out, alsa_cap, args.alsa_latency,
+			    stderr, debug);
+    }
 }
 #endif
 
@@ -1410,10 +1442,6 @@ grabber_init()
 {
     struct ng_video_fmt screen;
     void *base = NULL;
-#if defined(HAVE_ALSA)
-    void *md;
-    char *alsa_cap, *alsa_out, *p;
-#endif
 
     memset(&screen,0,sizeof(screen));
 #ifdef HAVE_LIBXXF86DGA
@@ -1458,25 +1486,33 @@ grabber_init()
 #if defined(HAVE_ALSA)
     if (args.alsa) {
 	/* Start audio capture thread */
-	md = discover_media_devices();
-	p = strrchr(args.device, '/');
+	void *md = discover_media_devices();
+	const char *p = strrchr(args.device, '/');
 	if (p)
 	    p++;
 	else
 	    p = args.device;
-	alsa_cap = (char *)get_associated_device(md, NULL, MEDIA_SND_CAP,
-						 p, MEDIA_V4L_VIDEO);
-	alsa_out = "default";
+
 	if (args.alsa_cap)
-		alsa_cap = args.alsa_cap;
+	    alsa_cap = args.alsa_cap;
+	else {
+	    p = get_associated_device(md, NULL, MEDIA_SND_CAP,
+				      p, MEDIA_V4L_VIDEO);
+	    if (p)
+		alsa_cap = strdup(p);
+	}
 
 	if (args.alsa_pb)
-		alsa_out = args.alsa_pb;
-	fprintf(stderr, "Alsa devices: cap: %s (%s), out: %s\n",
-		alsa_cap, args.device, alsa_out);
+	    alsa_out = args.alsa_pb;
+	else
+	    alsa_out = "default";
+
+	if (alsa_cap && alsa_out) {
+	    fprintf(stderr, "Alsa devices: cap: %s (%s), out: %s\n",
+		    alsa_cap, args.device, alsa_out);
+	    mute_notify = x11_mute_notify;
+	}
 
-	if (alsa_cap && alsa_out)
-	    alsa_thread_startup(alsa_out, alsa_cap, args.alsa_latency, stderr, debug);
 	free_media_devices(md);
     }
 #endif
@@ -1679,7 +1715,7 @@ usage(void)
 	    "      -(no)alsa       enable/disable alsa streaming. Default: enabled\n"
 	    "      -(no)alsa-cap   manually specify an alsa capture interface\n"
 	    "      -(no)alsa-pb    manually specify an alsa playback interface\n"
-	    "      -alsa-latency   manually specify an alsa latency in ms. Default: 30\n"
+	    "      -alsa-latency   manually specify an alsa latency in ms. Default: " STR(ALSA_LATENCY_DEFAULT) "\n"
 #endif
 	    "  -b  -bpp n          color depth of the display is n (n=24,32)\n"
 	    "  -o  -outfile file   filename base for snapshots\n"
